@@ -664,8 +664,7 @@ add_ext_keyuse_for_splitting(Dynamic_array<KEYUSE_EXT> *ext_keyuses,
   keyuse_ext.cond_guard= added_key_field->cond_guard;
   keyuse_ext.sj_pred_no= added_key_field->sj_pred_no;
   keyuse_ext.validity_ref= 0;
-  keyuse_ext.needed_in_prefix= added_key_field->val->used_tables() &
-			       ~(OUTER_REF_TABLE_BIT | RAND_TABLE_BIT);
+  keyuse_ext.needed_in_prefix= added_key_field->val->used_tables();
   keyuse_ext.validity_var= false;
   return ext_keyuses->push(keyuse_ext);
 }
@@ -766,7 +765,7 @@ double spl_postjoin_oper_cost(THD *thd, double join_record_count, uint rec_len)
 void JOIN::add_keyuses_for_splitting()
 {
   uint i;
-  uint idx;
+  size_t idx;
   KEYUSE_EXT *keyuse_ext;
   KEYUSE_EXT keyuse_ext_end;
   double oper_cost;
@@ -828,7 +827,7 @@ void JOIN::add_keyuses_for_splitting()
     added_keyuse->validity_ref= &keyuse_ext->validity_var;
   }
 
-  if (sort_and_filter_keyuse(thd, &keyuse, true))
+  if (sort_and_filter_keyuse(this, &keyuse, true))
     goto err;
   optimize_keyuse(this, &keyuse);
 
@@ -946,7 +945,6 @@ void reset_validity_vars_for_keyuses(KEYUSE_EXT *key_keyuse_ext_start,
 
 SplM_plan_info * JOIN_TAB::choose_best_splitting(uint idx,
                                                  table_map remaining_tables,
-                                                 const POSITION *join_positions,
                                                  table_map *spl_pd_boundary)
 {
   SplM_opt_info *spl_opt_info= table->spl_opt_info;
@@ -962,7 +960,7 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(uint idx,
   SplM_plan_info *spl_plan= 0;
   uint best_key= 0;
   uint best_key_parts= 0;
-  table_map best_param_tables= 0L;
+  table_map best_param_tables;
   Json_writer_object trace_obj(thd, "choose_best_splitting");
   Json_writer_array  trace_arr(thd, "considered_keys");
   /*
@@ -1044,7 +1042,7 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(uint idx,
     else
     {
       table_map last_found= this->table->map;
-      for (const POSITION *pos= &join_positions[idx - 1]; ; pos--)
+      for (POSITION *pos= &this->join->positions[idx - 1]; ; pos--)
       {
         if (pos->table->table->map & excluded_tables)
           continue;
