@@ -677,7 +677,7 @@ bool String::append_with_prefill(const char *s,uint32 arg_length,
 }
 
 
-int Binary_string::strstr(const char *search, uint32 search_length, uint32 offset) const
+int Binary_string::strstr(const char *search, uint32 search_length, uint32 offset)
 {
   if (search_length + offset <= str_length)
   {
@@ -703,7 +703,7 @@ skip:
   return -1;
 }
 
-int Binary_string::strstr(const Binary_string &s, uint32 offset) const
+int Binary_string::strstr(const Binary_string &s, uint32 offset)
 {
   return strstr(s.ptr(), s.length(), offset);
 }
@@ -712,7 +712,7 @@ int Binary_string::strstr(const Binary_string &s, uint32 offset) const
 ** Search string from end. Offset is offset to the end of string
 */
 
-int Binary_string::strrstr(const Binary_string &s, uint32 offset) const
+int Binary_string::strrstr(const Binary_string &s, uint32 offset)
 {
   if (s.length() <= offset && offset <= str_length)
   {
@@ -1156,28 +1156,22 @@ String_copier::well_formed_copy(CHARSET_INFO *to_cs,
   characters with backslashes as necessary.
   Does not add the enclosing quotes, this is left up to caller.
 */
-#define APPEND(...)   if (append(__VA_ARGS__)) return 1;
+#define APPEND(X)   if (append(X)) return 1; else break
 bool String::append_for_single_quote(const char *st, size_t len)
 {
   const char *end= st+len;
-  int chlen;
   for (; st < end; st++)
   {
-    switch (*st)
+    uchar c= *st;
+    switch (c)
     {
-    case '\\':   APPEND(STRING_WITH_LEN("\\\\")); break;
-    case '\0':   APPEND(STRING_WITH_LEN("\\0")); break;
-    case '\'':   APPEND(STRING_WITH_LEN("\\'")); break;
-    case '\n':   APPEND(STRING_WITH_LEN("\\n")); break;
-    case '\r':   APPEND(STRING_WITH_LEN("\\r")); break;
-    case '\032': APPEND(STRING_WITH_LEN("\\Z")); break;
-    default:     if ((chlen=charset()->charlen(st, end)) > 0)
-                 {
-                   APPEND(st, chlen);
-                   st+= chlen-1;
-                 }
-                 else
-                   APPEND(*st);
+    case '\\':   APPEND(STRING_WITH_LEN("\\\\"));
+    case '\0':   APPEND(STRING_WITH_LEN("\\0"));
+    case '\'':   APPEND(STRING_WITH_LEN("\\'"));
+    case '\n':   APPEND(STRING_WITH_LEN("\\n"));
+    case '\r':   APPEND(STRING_WITH_LEN("\\r"));
+    case '\032': APPEND(STRING_WITH_LEN("\\Z"));
+    default:     APPEND(c);
     }
   }
   return 0;
