@@ -473,7 +473,7 @@ btr_pcur_t::restore_position(btr_latch_mode restore_latch_mode, mtr_t *mtr)
 	if (rel_pos == BTR_PCUR_ON && btr_pcur_is_on_user_rec(this)) {
 		ulint n_matched_fields= 0;
 		if (!cmp_dtuple_rec_with_match(
-		      tuple, btr_pcur_get_rec(this),
+		      tuple, btr_pcur_get_rec(this), index,
 		      rec_get_offsets(btr_pcur_get_rec(this), index, offsets,
 			index->n_core_fields, ULINT_UNDEFINED, &heap),
 		      &n_matched_fields)) {
@@ -540,8 +540,7 @@ btr_pcur_move_to_next_page(
 
 	dberr_t err;
 	buf_block_t* next_block = btr_block_get(
-		*cursor->index(), next_page_no,
-		rw_lock_type_t(cursor->latch_mode & (RW_X_LATCH | RW_S_LATCH)),
+		*cursor->index(), next_page_no, cursor->latch_mode & ~12,
 		page_is_leaf(page), mtr, &err);
 
 	if (UNIV_UNLIKELY(!next_block)) {
