@@ -331,7 +331,9 @@ trx_purge_add_undo_to_history(const trx_t* trx, trx_undo_t*& undo, mtr_t* mtr)
 		/* Update the latest MySQL binlog name and offset info
 		in rollback segment header if MySQL binlogging is on
 		or the database server is a MySQL replication save. */
-		trx_rseg_update_binlog_offset(rseg_header, trx, mtr);
+		trx_rseg_update_binlog_offset(
+			rseg_header, trx->mysql_log_file_name,
+			trx->mysql_log_offset, mtr);
 	}
 
 	/* Add the log as the first in the history list */
@@ -369,6 +371,19 @@ trx_purge_add_undo_to_history(const trx_t* trx, trx_undo_t*& undo, mtr_t* mtr)
 	undo = NULL;
 }
 
+MY_ATTRIBUTE((nonnull, warn_unused_result))
+/** Remove undo log header from the history list.
+@param[in,out]  rseg    rollback segment header page
+@param[in]      log     undo log segment header page
+@param[in]      offset  byte offset in the undo log segment header page
+@param[in,out]  mtr     mini-transaction */
+static dberr_t trx_purge_remove_log_hdr(buf_block_t *rseg, buf_block_t* log,
+                                        uint16_t offset, mtr_t *mtr)
+{
+  return flst_remove(rseg, TRX_RSEG + TRX_RSEG_HISTORY, log,
+                     uint16_t(offset + TRX_UNDO_HISTORY_NODE), mtr);
+}
+
 /** Free an undo log segment.
 @param block     rollback segment header page
 @param mtr       mini-transaction */
@@ -378,7 +393,7 @@ static void trx_purge_free_segment(buf_block_t *block, mtr_t &mtr)
                                     block->page.frame, &mtr))
   {
     block->fix();
-    ut_d(const page_id_t id{block->page.id()});
+    const page_id_t id{block->page.id()};
     mtr.commit();
     /* NOTE: If the server is killed after the log that was produced
     up to this point was written, and before the log from the mtr.commit()
@@ -390,8 +405,16 @@ static void trx_purge_free_segment(buf_block_t *block, mtr_t &mtr)
     log_free_check();
     mtr.start();
     block->page.lock.x_lock();
-    ut_ad(block->page.id() == id);
-    mtr.memo_push(block, MTR_MEMO_PAGE_X_MODIFY);
+    if (UNIV_UNLIKELY(block->page.id() != id))
+    {
+      block->unfix();
+      block->page.lock.x_unlock();
+      block= buf_page_get_gen(id, 0, RW_X_LATCH, nullptr, BUF_GET, &mtr);
+      if (!block)
+        return;
+    }
+    else
+      mtr.memo_push(block, MTR_MEMO_PAGE_X_MODIFY);
   }
 
   while (!fseg_free_step(TRX_UNDO_SEG_HDR + TRX_UNDO_FSEG_HEADER +
@@ -399,13 +422,12 @@ static void trx_purge_free_segment(buf_block_t *block, mtr_t &mtr)
 }
 
 /** Remove unnecessary history data from a rollback segment.
-@param rseg   rollback segment
-@param limit  truncate anything before this
-@param all    whether everything can be truncated
+@param[in,out]	rseg		rollback segment
+@param[in]	limit		truncate anything before this
 @return error code */
 static dberr_t
-trx_purge_truncate_rseg_history(trx_rseg_t &rseg,
-                                const purge_sys_t::iterator &limit, bool all)
+trx_purge_truncate_rseg_history(trx_rseg_t& rseg,
+                                const purge_sys_t::iterator& limit)
 {
   fil_addr_t hdr_addr;
   mtr_t mtr;
@@ -414,6 +436,7 @@ trx_purge_truncate_rseg_history(trx_rseg_t &rseg,
   mtr.start();
 
   dberr_t err;
+reget:
   buf_block_t *rseg_hdr= rseg.get(&mtr, &err);
   if (!rseg_hdr)
   {
@@ -448,24 +471,23 @@ loop:
     goto func_exit;
   }
 
-  if (!all)
-    goto func_exit;
-
   fil_addr_t prev_hdr_addr=
     flst_get_prev_addr(b->page.frame + hdr_addr.boffset +
                        TRX_UNDO_HISTORY_NODE);
   prev_hdr_addr.boffset= static_cast<uint16_t>(prev_hdr_addr.boffset -
                                                TRX_UNDO_HISTORY_NODE);
-
-  err= flst_remove(rseg_hdr, TRX_RSEG + TRX_RSEG_HISTORY, b,
-                   uint16_t(hdr_addr.boffset + TRX_UNDO_HISTORY_NODE), &mtr);
+  err= trx_purge_remove_log_hdr(rseg_hdr, b, hdr_addr.boffset, &mtr);
   if (UNIV_UNLIKELY(err != DB_SUCCESS))
     goto func_exit;
 
   rseg_hdr->fix();
 
-  if (mach_read_from_2(b->page.frame + hdr_addr.boffset + TRX_UNDO_NEXT_LOG))
-    /* We cannot free the entire undo log segment. */;
+  if (mach_read_from_2(b->page.frame + hdr_addr.boffset + TRX_UNDO_NEXT_LOG) ||
+      rseg.is_referenced() ||
+      rseg.needs_purge > (purge_sys.head.trx_no
+                          ? purge_sys.head.trx_no
+                          : purge_sys.tail.trx_no))
+    /* We cannot free the entire undo page. */;
   else
   {
     const uint32_t seg_size=
@@ -490,9 +512,9 @@ loop:
         if (undo->hdr_page_no == hdr_addr.page)
           goto found_cached;
       ut_ad("inconsistent undo logs" == 0);
-      if (false)
-      found_cached:
-        UT_LIST_REMOVE(rseg.undo_cached, undo);
+      break;
+    found_cached:
+      UT_LIST_REMOVE(rseg.undo_cached, undo);
       static_assert(FIL_NULL == 0xffffffff, "");
       if (UNIV_UNLIKELY(mach_read_from_4(TRX_RSEG + TRX_RSEG_FORMAT +
                                          rseg_hdr->page.frame)))
@@ -515,7 +537,12 @@ loop:
   log_free_check();
   mtr.start();
   rseg_hdr->page.lock.x_lock();
-  ut_ad(rseg_hdr->page.id() == rseg.page_id());
+  if (UNIV_UNLIKELY(rseg_hdr->page.id() != rseg.page_id()))
+  {
+    rseg_hdr->unfix();
+    rseg_hdr->page.lock.x_unlock();
+    goto reget;
+  }
   mtr.memo_push(rseg_hdr, MTR_MEMO_PAGE_X_MODIFY);
 
   goto loop;
@@ -588,10 +615,7 @@ TRANSACTIONAL_TARGET static void trx_purge_truncate_history()
     {
       ut_ad(rseg.is_persistent());
       rseg.latch.wr_lock(SRW_LOCK_CALL);
-      if (dberr_t e=
-          trx_purge_truncate_rseg_history(rseg, head,
-                                          !rseg.is_referenced() &&
-                                          rseg.needs_purge <= head.trx_no))
+      if (dberr_t e= trx_purge_truncate_rseg_history(rseg, head))
         err= e;
       rseg.latch.wr_unlock();
     }
@@ -605,11 +629,11 @@ TRANSACTIONAL_TARGET static void trx_purge_truncate_history()
     {
       const ulint threshold=
         ulint(srv_max_undo_log_size >> srv_page_size_shift);
-      for (ulint i= purge_sys.truncate.last
+      for (uint32_t i= purge_sys.truncate.last
            ? purge_sys.truncate.last->id - srv_undo_space_id_start : 0,
            j= i;; )
       {
-        const auto space_id= srv_undo_space_id_start + i;
+        const uint32_t space_id= srv_undo_space_id_start + i;
         ut_ad(srv_is_undo_tablespace(space_id));
         fil_space_t *space= fil_space_get(space_id);
         ut_a(UT_LIST_GET_LEN(space->chain) == 1);
@@ -670,8 +694,7 @@ not_free:
       }
 
       ut_ad(rseg.curr_size > cached);
-      if (rseg.curr_size > cached + 1 &&
-          (rseg.history_size || srv_fast_shutdown || srv_undo_sources))
+      if (rseg.curr_size > cached + 1)
         goto not_free;
 
       rseg.latch.rd_unlock();
@@ -685,7 +708,6 @@ not_free:
     mtr_t mtr;
     mtr.start();
     mtr.x_lock_space(&space);
-    const auto space_id= space.id;
 
     /* Lock all modified pages of the tablespace.
 
@@ -695,8 +717,8 @@ not_free:
     mini-transaction commit and the server was killed, then
     discarding the to-be-trimmed pages without flushing would
     break crash recovery. */
-  rescan:
     mysql_mutex_lock(&buf_pool.flush_list_mutex);
+
     for (buf_page_t *bpage= UT_LIST_GET_LAST(buf_pool.flush_list); bpage; )
     {
       ut_ad(bpage->oldest_modification());
@@ -704,47 +726,46 @@ not_free:
 
       buf_page_t *prev= UT_LIST_GET_PREV(list, bpage);
 
-      if (bpage->oldest_modification() > 2 && bpage->id().space() == space_id)
+      if (bpage->id().space() == space.id &&
+          bpage->oldest_modification() != 1)
       {
         ut_ad(bpage->frame);
-        bpage->fix();
+        auto block= reinterpret_cast<buf_block_t*>(bpage);
+        if (!bpage->lock.x_lock_try())
         {
-          /* Try to acquire an exclusive latch while the cache line is
-          fresh after fix(). */
-          const bool got_lock{bpage->lock.x_lock_try()};
-          buf_pool.flush_hp.set(prev);
+        rescan:
+          /* Let buf_pool_t::release_freed_page() proceed. */
           mysql_mutex_unlock(&buf_pool.flush_list_mutex);
-          if (!got_lock)
-            bpage->lock.x_lock();
+          mysql_mutex_lock(&buf_pool.mutex);
+          mysql_mutex_lock(&buf_pool.flush_list_mutex);
+          mysql_mutex_unlock(&buf_pool.mutex);
+          bpage= UT_LIST_GET_LAST(buf_pool.flush_list);
+          continue;
         }
+        buf_pool.flush_hp.set(prev);
+        mysql_mutex_unlock(&buf_pool.flush_list_mutex);
 
 #ifdef BTR_CUR_HASH_ADAPT
-        /* There is no AHI on undo tablespaces. */
-        ut_ad(!reinterpret_cast<buf_block_t*>(bpage)->index);
+        ut_ad(!block->index); /* There is no AHI on undo tablespaces. */
 #endif
+        bpage->fix();
         ut_ad(!bpage->is_io_fixed());
-        ut_ad(bpage->id().space() == space_id);
+        mysql_mutex_lock(&buf_pool.flush_list_mutex);
 
-        if (bpage->oldest_modification() > 2)
+        if (bpage->oldest_modification() > 1)
         {
-          mtr.memo_push(reinterpret_cast<buf_block_t*>(bpage),
-                        MTR_MEMO_PAGE_X_FIX);
-          mysql_mutex_lock(&buf_pool.flush_list_mutex);
-          ut_ad(bpage->oldest_modification() > 2);
           bpage->reset_oldest_modification();
+          mtr.memo_push(block, MTR_MEMO_PAGE_X_FIX);
         }
         else
         {
           bpage->unfix();
           bpage->lock.x_unlock();
-          mysql_mutex_lock(&buf_pool.flush_list_mutex);
         }
 
         if (prev != buf_pool.flush_hp.get())
-        {
-          mysql_mutex_unlock(&buf_pool.flush_list_mutex);
+          /* Rescan, because we may have lost the position. */
           goto rescan;
-        }
       }
 
       bpage= prev;
